#!/usr/bin/python3
# -*- coding: utf-8 -*-
__author__ = "huangsheng"

import os
import sys
# reload(sys)
# sys.setdefaultencoding('utf8')
from PyQt5.QtCore import QSettings, QObject
from PyQt5.QtCore import pyqtSignal
from ftplib import FTP, error_perm
from hashlib import md5
from utils.common_utils import CommonUtils
from utils.access_thread import AccessThread
from utils.enums import AUPSource, Signals, AptFunc, RUNLIST, LaunchMode, HOMECACHE, \
    IMPORTANT_PATH, CRUCIAL_PATH
from model.upackage import UPackage
from utils.enums import Signals, AppType, AptFunc,iname_list,cname_list
from utils.log_manager import LogManager
import socket
import urllib
import urllib.request
import urllib.parse
import re
import getpass
import shutil
import time
from xdg import BaseDirectory as xdg


import gettext
gettext.textdomain("kylin-update-manager")
_ = gettext.gettext

class KylinUpdateService(QObject, Signals):
    oneAppmsg = pyqtSignal(dict)
    # check_os_update_over = pyqtSignal(name="check_os_update_over")
    KUM_CONF_PATH = "/etc/kum.conf"
    UPDATE_INTRANET_URL = "archive.kylinos-intranet.cn"
    UPDATE_INTERNET_URL = "archive.kylinos.cn"
    UPDATE_FILE_PATH = "/kylin/KYLIN-ALL/upgrades"

    SSMAP_PATH = os.path.join(xdg.xdg_cache_home, "kylin-update-manager/ss.map")
    SINFO_PATH= os.path.join(xdg.xdg_cache_home, "kylin-update-manager/sinfo")
    BACKUP_URL = "192.168.137.137"

    available_url = ""
    available_source = ""
    task_mode = ""
    sinfo_file_name = ""
    is_sinfo_match = ""
    is_sinfo_down_success = ""
    os_version_source = ""
    os_version = ""
    os_codename = ""
    desktop_or_server = ""

    ftp = None
    mw = None
    at = None
    # signal = None
    is_create = False


    is_error = False
    def __init__(self, mw):
        QObject.__init__(self)

        # ftp等库的超时时间
        socket.setdefaulttimeout(8)
        self.available_source = AUPSource.NOSOURCE
        self.mw = mw
        self.at = AccessThread(self)
        # self.signal = Signals(self)
        self.is_create = True
        self.at.accessOver.connect(self.mw.slot_access_completed)
        self.at.oneAppMsg.connect(self.mw.slot_get_one_app)
        self.oneAppmsg.connect(self.mw.slot_get_one_app)
        self.at.get_kylin_update_over.connect(self.slot_check_os_update_over)
        self.check_os_update_over.connect(self.mw.switch_task)
        # self.connect(self.at, Signals.get_kylin_update_over, self.slot_check_os_update_over)
        # self.connect(self, self.check_os_update_over, self.mw.switch_task)
        self.at.start()

        #self.desktop_or_server = mw.cu.desktop_or_server()
        #self.os_version = mw.cu.get_os_version()
        #self.os_codename = mw.cu.get_os_codename()
        #self.init_conf()

    def __del__(self):
        if(self.ftp != None):
            self.ftp.quit()

    #
    # 函数：读取/etc/kum.conf
    #
    def init_conf(self):
        if (os.path.isfile(self.KUM_CONF_PATH)):
            settings = QSettings(self.KUM_CONF_PATH, QSettings.IniFormat)
            settings.beginGroup("server_url")

            self.UPDATE_INTERNET_URL = str(settings.value("internet")).strip()
            self.UPDATE_INTRANET_URL = str(settings.value("intranet")).strip()
            self.BACKUP_URL = str(settings.value("backup_ip")).strip()

    #
    # 函数：检测是否有可用的升级媒介
    #
    def check_available_medium(self):
        # 清除旧文件
        self.clean_tmp()

        # 从各个媒介中寻找可用的源，下载ss.map和sinfo
        if(self.check_medium_cdrom() == True):
            self.download_ssmap_from_cdrom()
            self.is_sinfo_match = self.get_matched_conf_file()
            self.is_sinfo_down_success = self.download_sinfo_from_cdrom()

        elif(self.check_medium_intranet_server() == True):
            self.download_ssmap_from_intranet()
            self.is_sinfo_match = self.get_matched_conf_file()
            self.is_sinfo_down_success = self.download_sinfo_from_intranet()

        elif(self.check_medium_internet_server() == True):
            self.download_ssmap_from_internet()
            self.is_sinfo_match = self.get_matched_conf_file()
            self.is_sinfo_down_success = self.download_sinfo_from_internet()

        else:
            return False
        return True

    #
    # 函数：删除可能残留的旧ss.map和sinfo
    #
    def clean_tmp(self):
        if(os.path.exists(self.SSMAP_PATH)):
            os.remove(self.SSMAP_PATH)
        if(os.path.exists(self.SINFO_PATH)):
            os.remove(self.SINFO_PATH)

    #
    # 函数：检测升级光盘是否可用
    #
    def check_medium_cdrom(self):
        mount_root = "/media/"
        result = self.recursive_dirs(mount_root)
        if result == True:
            return True
        else:
            return False

    #
    # 函数：递归遍历目录，找到软件源
    #
    def recursive_dirs(self, dir):
        if os.path.isdir(dir):
            try:
                mount_dirs = os.listdir(dir)
            except Exception as e:
                # 跳过权限异常的目录，继续扫描下一个
                LogManager.ERR(e)
                # print(e)
                return False
            for one_dir in mount_dirs:
                one_dir_path = os.path.join(dir, one_dir)
                if os.path.isdir(one_dir_path) == False:
                    if os.path.basename(one_dir_path) == "ss.map":
                        self.available_url = "file://" + dir
                        self.available_source = AUPSource.DVDSOURCE
                        # print "找到光盘软件源."
                        LogManager.INFO(_("find CD Source."))
                        print(_("find CD Source."))
                        return True
                else:
                    result = self.recursive_dirs(one_dir_path)
                    if result == True:
                        return True
                    else:
                        continue

    #
    # 函数：检测局域网升级服务器是否可用
    #
    def check_medium_intranet_server(self):
        self.init_conf() #修改了kum.cof文件点击二次扫描后需要重新获取
        if(self.check_one_server(self.UPDATE_INTRANET_URL) == True):
            #print "通过DNS找到局域网软件源."
            LogManager.INFO(_("find LAN Source by DNS."))
            print(_("find LAN Source by DNS."))
            return True
        else:
            if(self.check_one_server(self.BACKUP_URL) == True):
                #print("通过IP找到局域网软件源.")
                LogManager.INFO(_("find LAN Source by IP."))
                print(_("find LAN Source by IP."))
                return True
            else:
                return False

    #
    # 函数：检查服务器
    #
    def check_one_server(self, url):
        try:
            if ":" in str(url):
                url_ip = str(url).split(":")[0]
                url_port = int(str(url).split(":")[1])
            else:
                url_ip = url
                url_port = 21
            # print("ip:",url_ip,"prot:",url_port)
            self.ftp = FTP()
            self.ftp.connect(host=url_ip, port=url_port)
            self.ftp.login("", "")
            self.available_url = "ftp://" + url
            self.available_source = AUPSource.FTPSOURCE
            return True
        except:
            http_url = "http://" + url
            try:
                urllib.request.urlopen(http_url + "/ss.map")
                self.available_url = http_url
                self.available_source = AUPSource.FTPSOURCE
                return True
            except:
                return False

    #
    # 函数：检测外网软件源是否可用
    #
    def check_medium_internet_server(self):
        url = "http://" + self.UPDATE_INTERNET_URL
        try:
            # kord源已开放，无需密码
            urllib.request.urlopen(url + self.UPDATE_FILE_PATH + "/ss.map")
            self.available_url = url
            self.available_source = AUPSource.HTTPSOURCE
            #print "找到互联网软件源."
            LogManager.INFO(_("find Internet Source."))
            print(_("find Internet Source."))
            return True
        except Exception as e:
            LogManager.ERR(e)
            # print(e)
            return False

    #
    # 函数：从光盘下载ss.map
    #
    def download_ssmap_from_cdrom(self):
        try:
            shutil.copyfile(self.available_url[7:] + "/ss.map", self.SSMAP_PATH)
        except:
            self.mw.loading_widget.change_layout.emit(_("No software source available"))
            LogManager.INFO(_("拷贝光盘源ss.map文件失败"))
            # self.mw.clean_sp_widget.emit()RR

    #
    # 函数：从局域网服务器下载ss.map
    #
    def download_ssmap_from_intranet(self):
        bufsize = 1024
        try:
            fp = open(self.SSMAP_PATH, "wb")
            self.ftp.retrbinary("RETR " + "pub/kylin/ss.map", fp.write, bufsize)
            fp.close()
        except:
            try:
                url = self.available_url + "/ss.map"
                urllib.request.urlretrieve(url, self.SSMAP_PATH)
            except:
                # self.mw.signal_show_exit.emit(_("No software source available"))
                # print("下载ss.map文件失败")
                LogManager.INFO(_("局域网源下载ss.map文件失败"))
                self.mw.loading_widget.change_layout.emit(_("No software source available"))
                # self.mw.clean_sp_widget.emit()

    #
    # 函数：从互联网服务器下载ss.map
    #
    def download_ssmap_from_internet(self):
        # url = "http://" + self.mw.service_key + ":" + self.mw.hash_pwd + "@" + self.UPDATE_INTERNET_URL + "/ss.map"
        # kord源已开放，无需密码
        try:
            url = "http://" + self.UPDATE_INTERNET_URL + self.UPDATE_FILE_PATH + "/ss.map"
            urllib.request.urlretrieve(url, self.SSMAP_PATH)
        except:
            # self.mw.signal_show_exit.emit(_("No software source available"))
            self.mw.loading_widget.change_layout.emit(_("No software source available"))
            LogManager.INFO(_("外网源下载ss.map文件失败"))
            # self.mw.clean_sp_widget.emit()

    #
    # 函数：获得和本地系统匹配的软件源配置文件名
    #
    def get_matched_conf_file(self):
        settings = QSettings(self.SSMAP_PATH, QSettings.IniFormat)
        settings.beginGroup("key-source")
        keys = settings.childKeys()
        if(self.mw.service_key == "kylin"):
            # 社区版，直接使用默认sinfo
            self.sinfo_file_name = "sinfo.standard." + self.os_codename
            return True
        else:
            # 商业版，匹配sinfo
            for key in keys:
                if(str(key) == self.mw.service_key):
                    self.sinfo_file_name = "sinfo." + str(settings.value(key)).strip()
                    return True
            else:
                # 没有匹配的文件，使用默认值
                self.sinfo_file_name = "sinfo.standard." + self.os_codename
                return False

    #
    # 函数：从光盘下载sinfo
    #
    def download_sinfo_from_cdrom(self):
        if (os.path.exists(self.SINFO_PATH)):
            cmd = "rm " + self.SINFO_PATH
            os.system(cmd)
        try:
            # print(self.available_url[7:] + "/" + self.sinfo_file_name, self.SINFO_PATH)
            if os.path.exists(HOMECACHE):
                print(self.available_url[7:])
                shutil.copyfile(self.available_url[7:] + "/" + self.sinfo_file_name, self.SINFO_PATH)
            else:
                cmd = "mkdir " + HOMECACHE
                os.system(cmd)
            return True
        except Exception as e:
            print(_("copy error : "), e)
            LogManager.INFO(_("光盘源源拷贝sinfo文件失败：%e" % str(e)))
            return False

    #
    # 函数：从局域网服务器下载sinfo
    #
    def download_sinfo_from_intranet(self):
        fp = open(self.SINFO_PATH, "wb")
        try:
            bufsize = 1024
            self.ftp.retrbinary("RETR " + "pub/kylin/" + self.sinfo_file_name, fp.write, bufsize)
            fp.close()
            return True
        except:
            if (os.path.exists(self.SINFO_PATH)):
                cmd = "rm " + self.SINFO_PATH
                os.system(cmd)
            url = self.available_url
            try:
                ##kord源已开放，无需密码
                f = urllib.request.urlopen(url + "/" + self.sinfo_file_name)
                with open(self.SINFO_PATH, "wb") as code:
                    code.write(f.read())
                return True
            except Exception as e:
                LogManager.ERR("局域网源下载sinfo文件失败：%s" % str(e))
                return False
            # print(_("download error : "), e)

    #
    # 函数：从互联网服务器下载sinfo
    #
    def download_sinfo_from_internet(self):
        if (os.path.exists(self.SINFO_PATH)):
            cmd = "rm " + self.SINFO_PATH
            os.system(cmd)
        url = "http://" + self.UPDATE_INTERNET_URL + self.UPDATE_FILE_PATH
        try:
            # password_mgr = urllib3.HTTPPasswordMgrWithDefaultRealm()
            # password_mgr.add_password(None, url, self.mw.service_key, self.mw.hash_pwd)
            # opener = urllib3.build_opener(urllib3.HTTPBasicAuthHandler(password_mgr))
            # urllib3.install_opener(opener)
            ##kord源已开放，无需密码
            f = urllib.request.urlopen(url + "/" + self.sinfo_file_name)
            with open(self.SINFO_PATH, "wb") as code:
                code.write(f.read())
            return True

        except Exception as e:
            LogManager.ERR("外网源下载sinfo文件失败：%s" % str(e))
            # print(e)
            return False

    #
    # 函数：判断是否需要apt-get update
    #
    def check_need_apt_get_update(self):
        # md5s_source = []
        # start = False
        # try:
        #     for i in range(6):
        #         try:
        #             fp = open(self.SINFO_PATH, "r")
        #             lines = fp.readlines()
        #             for line in lines:
        #                 if (start == True):
        #                     if (line.find("[") != -1 or line.strip() == "" or line == None):
        #                         break
        #                     else:
        #                         md5s_source.append(line.strip())
        #                 if (line.find("[inrelease.md5]") != -1):
        #                     start = True
        #             fp.close()
        #             md5s_local = self.get_local_inrelease_md5()
        #             print("source:",md5s_source)
        #             #/var/lib/apt/lists下面有可能为空，导致遍历有问题，此时需要update—lj
        #             if len(md5s_source) == 0:
        #                 return True
        #             for md5_source in md5s_source:
        #                 for md5_local in md5s_local:
        #                     if (md5_source == md5_local):
        #                         print("not need update")
        #                         return False
        #             # 不需要apt-get update
        #             return True
        #         except:
        #             time.sleep(0.5)
        # except:
        #     if self.mw.launch_mode == LaunchMode.AUTOMATIC:
        #         self.mw.exit_app()
        #     #self.mw.show_exit_widget(_("Missing sinfo file"))
        #     self.mw.loading_widget.change_layout.emit(_("Missing sinfo file"))
        #     # self.mw.clean_sp_widget.emit()
        #     pass
        return True



    #
    # 函数：获取下载的sources.list
    #
    def get_sources(self):
        sourceslist = []
        start = False
        try:
            for i in range(6):
                try:
                    fp = open(self.SINFO_PATH, "r")
                    lines = fp.readlines()
                    for line in lines:
                        if (start == True):
                            if (line.find("[") != -1 or line.strip() == "" or line == None):
                                break
                            else:
                                # kord源已开放，无需密码
                                sourceslist.append("deb " + self.available_url + line.strip())
                        if (line.find("[sources.list]") != -1):
                            start = True
                    fp.close()
                    return sourceslist
                except:
                    time.sleep(0.5)
            #fp = open(self.SINFO_PATH, "r")

        except:
            if self.mw.launch_mode == LaunchMode.AUTOMATIC:
                self.mw.exit_app()
            #self.mw.show_exit_widget(_("Missing sinfo file"))
            self.mw.loading_widget.change_layout.emit(_("Missing sinfo file"))
            # self.mw.clean_sp_widget.emit()
            pass

    #
    # 函数：获取下载的important和crucial列表
    #
    def get_lists(self):
        start = False
        while True:
            if os.path.isfile(self.SINFO_PATH):
                break
            else:
                pass
        try:
            for i in range(6):
                try:
                    fp = open(self.SINFO_PATH, "r")
                    lines = fp.readlines()
                    for line in lines:
                        # print("sinfo行:", line)
                        if (start == True):
                            if (line.find("[") != -1 or line.strip() == "" or line == None):
                                break
                            else:
                                if ("," in line):  # 字段含逗号，说明逗号后面是运行命令
                                    line = line.split(",")
                                    if (line[1].strip() == "run"):
                                        RUNLIST.append(line[0].strip())  # 逗号后面是run表示用软件名运行
                                    else:
                                        RUNLIST.append(line[1].strip())  # 否则逗号后面是运行命令
                                    # print("get_lists333",line[0].strip())
                                    iname_list.append(line[0].strip())
                                else:
                                    # print("get_lists444",line.strip())
                                    iname_list.append(line.strip())

                        if (line.find("[important." + self.desktop_or_server + "]") != -1):
                            start = True
                    fp.close()
                    break
                except:
                    time.sleep(0.5)
            #fp = open(self.SINFO_PATH, "r")

        except:
            if self.mw.launch_mode == LaunchMode.AUTOMATIC:
                self.mw.exit_app()
            #self.mw.show_exit_widget(_("Missing sinfo file"))
            self.mw.loading_widget.change_layout.emit(_("Missing sinfo file"))
            # self.mw.clean_sp_widget.emit()
            pass
        start = False
        try:
            for i in range(6):
                try:
                    fp = open(self.SINFO_PATH, "r")
                    lines = fp.readlines()
                    for line in lines:
                        if (start == True):
                            if (line.find("[") != -1 or line.strip() == "" or line == None):
                                break
                            else:
                                if ("," in line):  # 字段含逗号，说明逗号后面是运行命令
                                    line = line.split(",")
                                    if (line[1].strip() == "run"):
                                        RUNLIST.append(line[0].strip())  # 逗号后面是run表示用软件名运行
                                    else:
                                        RUNLIST.append(line[1].strip())  # 否则逗号后面是运行命令
                                    cname_list.append(line[0].strip())
                                else:
                                    cname_list.append(line.strip())

                        if (line.find("[crucial." + self.desktop_or_server + "]") != -1):
                            start = True
                    fp.close()
                except:
                    time.sleep(0.5)
            #fp = open(self.SINFO_PATH, "r")

        except:
            if self.mw.launch_mode == LaunchMode.AUTOMATIC:
                self.mw.exit_app()
            #self.mw.show_exit_widget(_("Missing sinfo file"))
            self.mw.loading_widget.change_layout.emit(_("Missing sinfo file"))
            # self.mw.clean_sp_widget.emit()
        return (iname_list, cname_list)

    #
    # 函数：读取源管理获取到的important和crucial文件内容
    #
    def get_software_properties_list(self):
        error_time = 0
        try:
            fp = open(IMPORTANT_PATH, "r")
            lines = fp.readlines()
            for line in lines:
                list = line.strip("\n").split(" ")
                iname_list.extend(list)
        except Exception as e:
            error_time += 1
            print("源服务器缺少important.list文件")
        try:
            fp = open(CRUCIAL_PATH, "r")
            lines = fp.readlines()
            for line in lines:
                list = line.strip("\n").split(" ")
                cname_list.extend(list)
        except Exception as e:
            if error_time == 1:
                self.mw.loading_widget.change_layout.emit(_("Missing important.list and crucial.list file"))
            else:
                print("源服务器缺少crucial.list文件")
        return (iname_list, cname_list)

    #
    # 函数：获取本地InRelease文件md5值
    #
    def get_local_inrelease_md5(self):
        lists_path = "/var/lib/apt/lists"
        files = os.listdir(lists_path)
        md5s_local = []

        for f in files:
            full_path = lists_path + "/" + f
            if(os.path.isfile(full_path) == True):
                if(full_path.endswith("InRelease")):
                    ff = open(full_path, "rb")
                    m = md5()
                    m.update(ff.read())
                    md5s_local.append(m.hexdigest())

        return md5s_local

    #
    # 函数：开始apt任务，检测是否有自我升级、系统升级、以及软件包和补丁包升级
    #
    def check_update_type(self, state=False):
        #try:
        if state:
            (self.iname_list, self.cname_list) = self.get_software_properties_list()
        else:
            (self.iname_list, self.cname_list) = self.get_lists()
        self.at.work_list.append(AptFunc.INIT)
        self.at.work_list.append(AptFunc.GET_KYLIN_UPDATE)
        print("success")
        '''except:
            if self.mw.launch_mode == LaunchMode.AUTOMATIC:
                self.mw.exit_app()
            self.mw.show_exit_widget(_("Missing sinfo file"))'''

    #
    # 函数：判断是否有可用的系统升级和更新器自我升级完成
    # ku_list:SP升级包列表
    # kum：自升级包
    #
    def slot_check_os_update_over(self, ku_list, kum):
        is_kum_update = False
        if (kum != None):
            is_kum_update = kum.is_upgradable
            version_list = kum.versions
            version = str(version_list[0]).split("=")[1]
            #print(version)
            upkg = UPackage(kum,self.at.ca)
            self.at.fill_info_cn(upkg)
            if is_kum_update == True:
                self.oneAppmsg.emit(
                    {"apptype": "", "displayName": upkg.displayName, "name": upkg.name,
                     "icon": "./res/logo.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                     "downloadSize": upkg.downloadSize, "installedVersion": version,
                     "architecture": kum._pkg.architecture, "fullversion": "",
                     "candidateVersion": kum.candidate.version, "description": upkg.description})
                # 如果是自升级直接返回不需要再检测是否有sp升级-lj
                self.check_os_update_over.emit(False, is_kum_update)
                return
        if (len(ku_list) == 0):
            # 源中没有升级包
            self.check_os_update_over.emit(False, is_kum_update)
        elif ku_list[0].is_upgradable or ku_list[0].is_installed == False:
            for pkg in ku_list:
                name = pkg.name
                if (name.find(self.desktop_or_server) != -1):
                    version = pkg.candidate.version
                    osversion = version[:version.find('-', version.find('-') + 1)]
                    version_list = version.upper().split('-')
                    self.os_version_source = osversion.upper()
                    self.os_version = self.os_version.upper()
                    #大版本中系统版本中有字母仓库源版本中没有字母或系统版本没有字母仓库源版本中有字母
                    if((len(re.findall("[a-zA-Z]+",self.os_version.split('S')[0]))>0)):
                        system_version = re.sub("[a-zA-Z]+", "", self.os_version.split("S")[0]).strip("-")
                    else:
                        system_version = self.os_version.split('S')[0].strip("-")
                    if(len(re.findall("[a-zA-Z]+",self.os_version_source.split('S')[0]))>0):
                        src_version = re.sub("[a-zA-Z]+","",self.os_version_source.split('S')[0])
                    else:
                        src_version = self.os_version_source.split('S')[0].strip("-")
                    system_big_version = system_version[len(system_version) - 1]
                    if(src_version.startswith("10")):
                        src_big_version = src_version
                    else:
                        src_big_version = src_version[len(src_version) - 1]
                    #system_version:为系统中的版本号
                    #src_version:为仓库源中的版本号
                    # 大版本不一样
                    #从v4到v10
                    if(system_version.startswith("4")):
                        if(src_version.startswith("10")):
                            upkg = UPackage(pkg,self.at.ca)
                            new_version = osversion.upper()
                            self.at.fill_info_cn(upkg)
                            if is_kum_update == False:
                                self.oneAppmsg.emit(
                                    {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                     "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                     "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                     "architecture": kum._pkg.architecture, "fullversion": version,
                                     "candidateVersion": new_version, "description": upkg.description})
                            self.check_os_update_over.emit(True, is_kum_update)
                            break
                        else:
                            if(float(src_big_version) > float(system_big_version)):
                                upkg = UPackage(pkg,self.at.ca)
                                new_version = osversion.upper()
                                self.at.fill_info_cn(upkg)
                                if is_kum_update == False:
                                    self.oneAppmsg.emit(
                                        {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                         "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                         "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                         "architecture": kum._pkg.architecture, "fullversion": version,
                                         "candidateVersion": new_version, "description": upkg.description})
                                self.check_os_update_over.emit(True, is_kum_update)
                                break
                            else:
                                # 源中的系统升级包没有SP关键字
                                if (osversion.upper().find('SP') == -1):
                                    upkg = UPackage(pkg,self.at.ca)
                                    new_version = osversion.upper()
                                    self.at.fill_info_cn(upkg)
                                    if is_kum_update == False:
                                        self.oneAppmsg.emit(
                                            {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                             "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                             "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                             "architecture": kum._pkg.architecture, "fullversion": version,
                                             "candidateVersion": new_version, "description": upkg.description})
                                        self.check_os_update_over.emit(True, is_kum_update)
                                        break
                                else:
                                    os_version_source_numb = int(osversion[osversion.upper().find('SP') + 2:])
                                    if (self.os_version.upper().find("SP") != -1):
                                        os_version_numb = int(self.os_version[self.os_version.upper().find('SP') + 2:])
                                    else:
                                        os_version_numb = 0

                                    if (self.os_version.upper().find('SP') == -1 and os_version_source_numb > 0):
                                        # 找到可用的系统升级
                                        upkg = UPackage(pkg,self.at.ca)
                                        new_version = osversion.upper()
                                        self.at.fill_info_cn(upkg)
                                        if is_kum_update == False:
                                            self.oneAppmsg.emit(
                                                {"apptype": AppType.SP, "displayName": upkg.displayName,
                                                 "name": upkg.name,
                                                 "icon": "./res/SP_icon.png",
                                                 "downloadSizeHuman": upkg.downloadSizeHuman,
                                                 "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                                 "architecture": kum._pkg.architecture, "fullversion": version,
                                                 "candidateVersion": new_version, "description": upkg.description})
                                        self.check_os_update_over.emit(True, is_kum_update)
                                        # self.emit(self.check_os_update_over, True, is_kum_update)
                                        break
                                    elif (self.os_version.upper().find(
                                            'SP') != -1 and os_version_source_numb > os_version_numb):

                                        # 找到可用的系统升级
                                        upkg = UPackage(pkg,self.at.ca)
                                        new_version = osversion.upper()

                                        self.at.fill_info_cn(upkg)
                                        if is_kum_update == False:
                                            self.oneAppmsg.emit(
                                                {"apptype": AppType.SP, "displayName": upkg.displayName,
                                                 "name": upkg.name,
                                                 "icon": "./res/SP_icon.png",
                                                 "downloadSizeHuman": upkg.downloadSizeHuman,
                                                 "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                                 "architecture": kum._pkg.architecture, "fullversion": version,
                                                 "candidateVersion": new_version, "description": upkg.description})
                                        # self.emit(self.check_os_update_over, True, is_kum_update)
                                        self.check_os_update_over.emit(True, is_kum_update)
                                        break
                                    else:

                                        # 当前版本大于或等于源版本时，直接进行软件升级
                                        self.check_os_update_over.emit(False, is_kum_update)
                                        break
                    else:
                        if(float(src_big_version) < float(system_version)):
                            self.check_os_update_over.emit(False, is_kum_update)
                            break
                        elif(float(src_big_version) == float(system_version)):
                            # 源中的系统升级包没有SP关键字
                            if (osversion.upper().find('SP') == -1):
                                upkg = UPackage(pkg,self.at.ca)
                                new_version = osversion.upper()
                                self.at.fill_info_cn(upkg)
                                if is_kum_update == False:
                                    self.oneAppmsg.emit(
                                        {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                         "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                         "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                         "architecture": kum._pkg.architecture, "fullversion": version,
                                         "candidateVersion": new_version, "description": upkg.description})
                                    self.check_os_update_over.emit(True, is_kum_update)
                                    break
                            else:
                                os_version_source_numb = int(osversion[osversion.upper().find('SP') + 2:])
                                if (self.os_version.upper().find("SP") != -1):
                                    os_version_numb = int(self.os_version[self.os_version.upper().find('SP') + 2:])
                                else:
                                    os_version_numb = 0
                                if (os_version_source_numb > os_version_numb):
                                    # 找到可用的系统升级
                                    upkg = UPackage(pkg,self.at.ca)
                                    new_version = osversion.upper()
                                    self.at.fill_info_cn(upkg)
                                    if is_kum_update == False:
                                        self.oneAppmsg.emit(
                                            {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                             "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                             "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                             "architecture": kum._pkg.architecture, "fullversion": version,
                                             "candidateVersion": new_version, "description": upkg.description})
                                    self.check_os_update_over.emit(True, is_kum_update)
                                    # self.emit(self.check_os_update_over, True, is_kum_update)
                                    break
                                else:

                                    # 当前版本大于或等于源版本时，直接进行软件升级
                                    self.check_os_update_over.emit(False, is_kum_update)
                                '''else:
                                    # self.emit(self.check_os_update_over, False, is_kum_update)
                                    self.check_os_update_over.emit(False, is_kum_update)
                                    pass'''
                        #大版本小于源里面的版本
                        else:
                            upkg = UPackage(pkg, self.at.ca)
                            new_version = osversion.upper()
                            self.at.fill_info_cn(upkg)
                            if is_kum_update == False:
                                self.oneAppmsg.emit(
                                    {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                     "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                     "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                     "architecture": kum._pkg.architecture, "fullversion": version,
                                     "candidateVersion": new_version, "description": upkg.description})
                                self.check_os_update_over.emit(True, is_kum_update)
                                break
                else:
                    # 源中没有系统升级包 / 本地和源不是同一个大版本 / 源中没有可更新的版本
                    # self.emit(self.check_os_update_over, False, is_kum_update)
                    self.check_os_update_over.emit(False, is_kum_update)
        else:
            self.check_os_update_over.emit(False, False)

    #
    # 函数：获取系统本地版本和可升级的版本
    #
    def get_os_versions(self):
        return (self.os_version, self.os_version_source)

    #
    # 函数：开始获取和分类各种升级包
    #
    def start_access(self):
        self.at.work_list.append(AptFunc.GET_ALL_UPDATE)

    #
    # 函数：获取的系统patch版本
    #
    def get_patch_version(self):
        path = "/etc/kylin-build"
        with open(path,"r") as f:
            lines = f.readlines()
            if(len(lines) != 0):
                lastline = lines[len(lines) - 1]
                if "kylin-update-patch" in lastline:
                    current_version = lastline.split(" ")[1].split("~")[1]
                    if(len(current_version) == 4):
                        current_version = "2020" + current_version
                else:
                    current_version = "20200710"
                return current_version


def main():
    class mw:
        cu = CommonUtils()
        service_key = "750247"
        hash_pwd = "7f5831fbbb60350cb32e39d337fc7c22"
    # kus.clean_tmp()
    # print kus.check_medium_cdrom()
    # print kus.available_url
    # kus.download_ssmap_from_cdrom()
    # kus.get_matched_conf_file()
    # print kus.sinfo_file_name
    # print kus.download_sinfo_from_cdrom()

    # print kus.check_available_medium()
    # print kus.sinfo_file_name
    # print "is match : ", kus.is_sinfo_match
    # print "is down success : ", kus.is_sinfo_down_success
    # print "is need apt update : ", kus.check_need_apt_get_update()
    # print kus.get_sources()


if __name__ == '__main__':
    main()
